﻿using System;
using System.Linq;
using linq = System.Linq.Expressions;
using VirtoCommerce.Foundation.Catalogs.Services;
using VirtoCommerce.Foundation.Frameworks;
using VirtoCommerce.ManagementClient.Core.Controls;
using VirtoCommerce.ManagementClient.Core.Infrastructure;
using System.Reflection;

namespace VirtoCommerce.ManagementClient.Catalog.Model.TypedExpressions.Conditions
{
	[Serializable]
	public class ConditionCurrentUrlIs : TypedExpressionElementBase, IExpressionAdaptor
	{
		private MatchContainsCaseStringElement _urlEl;

		public ConditionCurrentUrlIs(IExpressionViewModel expressionViewModel)
			: base("Currently viewed Url []", expressionViewModel)
		{
			WithLabel("Current Url ");
			_urlEl = WithElement(new MatchContainsCaseStringElement(expressionViewModel)) as MatchContainsCaseStringElement;
		}

		public string Url
		{
			get
			{
				return _urlEl.ValueString;
			}
			set
			{
				_urlEl.ValueString = value;
			}
		}

		public string MatchConditionValue
		{
			get
			{
				return _urlEl.MatchingContains.InputValue.ToString();
			}
			set
			{
				_urlEl.MatchingContains.InputValue = value;
			}
		}

		public MatchingContains MatchCondition
		{
			get
			{
				return _urlEl.MatchingContains;
			}
		}

		public linq.Expression<Func<IEvaluationContext, bool>> GetExpression()
		{
			linq.ParameterExpression paramX = linq.Expression.Parameter(typeof(IEvaluationContext), "x");
			var castOp = linq.Expression.MakeUnary(linq.ExpressionType.Convert, paramX, typeof(PriceListAssignmentEvaluationContext));
			var propertyValue = linq.Expression.Property(castOp, typeof(PriceListAssignmentEvaluationContext).GetProperty("CurrentUrl"));
			
			MethodInfo method;
			linq.Expression methodExp;
			
			if (string.Equals(MatchConditionValue, MatchCondition.Contains))
			{
				method = typeof(string).GetMethod("Contains", new[] { typeof(string) });
				var toLowerMethod = typeof(string).GetMethod("ToLowerInvariant");
				var toLowerExp = linq.Expression.Call(propertyValue, toLowerMethod);
				methodExp = linq.Expression.Call(toLowerExp, method, linq.Expression.Constant(Url.ToLowerInvariant()));
			}
			else if (string.Equals(MatchConditionValue, MatchCondition.Matching))
			{
				method = typeof(string).GetMethod("Equals", new[] { typeof(string) });
				var toLowerMethod = typeof(string).GetMethod("ToLowerInvariant");
				var toLowerExp = linq.Expression.Call(propertyValue, toLowerMethod);
				methodExp = linq.Expression.Call(toLowerExp, method, linq.Expression.Constant(Url.ToLowerInvariant()));
			}
			else if (string.Equals(MatchConditionValue, MatchCondition.ContainsCase))
			{
				method = typeof(string).GetMethod("Contains", new[] { typeof(string) });
				methodExp = linq.Expression.Call(propertyValue, method, linq.Expression.Constant(Url));
			}
			else if (string.Equals(MatchConditionValue, MatchCondition.MatchingCase))
			{
				method = typeof(string).GetMethod("Equals", new[] { typeof(string) });
				methodExp = linq.Expression.Call(propertyValue, method, linq.Expression.Constant(Url));
			}
			else if (string.Equals(MatchConditionValue, MatchCondition.NotContains))
			{
				method = typeof(string).GetMethod("Contains", new[] { typeof(string) });
				var toLowerMethod = typeof(string).GetMethod("ToLowerInvariant");
				var toLowerExp = linq.Expression.Call(propertyValue, toLowerMethod);
				methodExp =  linq.Expression.Not(linq.Expression.Call(toLowerExp, method, linq.Expression.Constant(Url.ToLowerInvariant())));
			}
			else if (string.Equals(MatchConditionValue, MatchCondition.NotMatching))
			{
				method = typeof(string).GetMethod("Equals", new[] { typeof(string) });
				var toLowerMethod = typeof(string).GetMethod("ToLowerInvariant");
				var toLowerExp = linq.Expression.Call(propertyValue, toLowerMethod);
				methodExp =  linq.Expression.Not(linq.Expression.Call(toLowerExp, method, linq.Expression.Constant(Url.ToLowerInvariant())));
			}
			else if (string.Equals(MatchConditionValue, MatchCondition.NotContainsCase))
			{
				method = typeof(string).GetMethod("Contains", new[] { typeof(string) });
				methodExp =  linq.Expression.Not(linq.Expression.Call(propertyValue, method, linq.Expression.Constant(Url)));
			}
			else
			{
				method = typeof(string).GetMethod("Equals", new[] { typeof(string) });
				methodExp =  linq.Expression.Not(linq.Expression.Call(propertyValue, method, linq.Expression.Constant(Url)));
			}
			
			var retVal = linq.Expression.Lambda<Func<IEvaluationContext, bool>>(methodExp, paramX);

			//var retVal = linq.Expression.Lambda<Func<IEvaluationContext, bool>>(methodCall, paramX);

			return retVal;
		}
	}
}
